% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nlme_param_sampling.R
\name{detectSampleDiscrepancy}
\alias{detectSampleDiscrepancy}
\title{Detect a discrepancy between the empirical normal distribution of a sample of (transformed to normal scale)
NLME parameters and a specified "target" normal distribution.}
\usage{
detectSampleDiscrepancy(
  obj,
  spec = specifyParamSampling(obj),
  testSample = sampleParamFromUncertainty(spec, Npop = Npop * Ntests),
  targetMuNormal = unlist(transformParamToNormal(spec, getParamEstimates(spec))[1,
    getNamesAllParameters(spec)]),
  targetSigmaNormal = getCovMatrixUncertainty(spec),
  targetSample = untransformParamFromNormal(spec, as.data.frame(MASS::mvrnorm(Npop *
    Ntests, mu = targetMuNormal, Sigma = targetSigmaNormal))),
  Npop = 200,
  Ntests = 5,
  tAlpha = 0.01,
  FLAGverbose = FALSE
)
}
\arguments{
\item{obj}{a filename (character string) denoting the path to a GPF
file, or a GPF object, or an IQRnlmeParamSpec object. This argument is
ignored if \code{spec} is specified.}

\item{spec}{an IQRnlmeParamSpec object. This argument overwrites the argument \code{obj}.
Default: \code{specifyParamSampling(obj)}.}

\item{testSample}{a data.frame of \code{Ntests*Npop} rows with columns named as (some of) the parameters in
\code{spec} (see \code{\link{getNamesAllParameters}}) and rows corresponding to different sampled tupplets
of such parameters. The parameter values in this data.frame are at the
original scale (not transformed to normal scale). Default: \code{sampleParamFromUncertainty(spec, Npop = Npop*Ntests)}.}

\item{targetMuNormal}{a named numeric vector specifying the mean of the target normal distribution (on the normal scale).
Default: \code{unlist(transformParamToNormal(spec, getParamEstimates(spec))[1L, getNamesAllParameters(spec)])}}

\item{targetSigmaNormal}{a square matrix specifying the variance covariance matrix of the target normal distribution (on the normal scale).
Default: \code{getCovMatrixUncertainty(spec)}.}

\item{targetSample}{a data.frame of the same format as \code{testSample}.
Default: \code{untransformParamFromNormal(spec, as.data.frame(MASS::mvrnorm(Npop * Ntests, mu = targetMuNormal, Sigma = targetSigmaNormal)))}}

\item{Npop, Ntests}{integers defining the number of rows in one parameter sample for one T-test. The number of rows in
\code{testSample} and \code{targetSample} is equal to \code{Npop*Ntests}. If \code{testSample} and \code{targetSample} are
specified \code{Npop} is set to  \code{nrow(testSample) \%/\% Ntests}.}

\item{tAlpha}{a double between 0 and 1 specifying the critical p-value level for the T-tests. Note: p-values are Bonferoni corrected.}

\item{FLAGverbose}{a logical indicating if information should be printed to the console.}
}
\value{
A named list with at least the following elements:
\itemize{
\item detectedSingleParameters: a character vector of the names of detected parameters with apparent sampling discrepancy;
\item listHistogramsDetectedSingleParams: a list of ggplot objects;
\item detectedPairParameters: a character vector of the names of detected parameter-pairs with apparent sampling discrepancy;
\item listDensPlotsDetectedPairParams: a list of ggplot objects.
}
}
\description{
In brief, this function tests whether the Bhattacharyya distances to the normal distribution
defined by \code{targetMuNormal} and \code{targetSigmaNormal} of (the transformed to normal scale) values in
\code{testSample} and \code{targetSample} are significantly different. The Bhattacharyya distance measures the
overlap between two (multivariate) normal distributions. It is assumed that
\code{targetSample} is consistent with \code{targetMuNormal} and \code{targetSigmaNormal}. A discrepancy is then
reported for each parameter X (column X in \code{testSample} and \code{targetSample}), and for each pair
of parameters X,Y, such that the T-test Bonferoni-corrected p-value for differing mean Bhattacharyya distances is
significant at the level \code{tAlpha}. Specifically, for each parameter X and pair of parameters X,Y
among the variable parameter columns in \code{testSample} and \code{targetSample}, the procedure goes as follows:
\enumerate{
\item{Transform \code{testSample} and {targetSample} to normal scale based on the parameter transformation in \code{spec}.}
\item{Divide the two transformed samples into \code{Ntests} chunks of \code{Npop} rows.}
\item{Calculate the Bhattacharyya distances between the normal distributions defined by the empirical mean
vector and covariance matrix of each (test and target) chunk and the normal distribution defined by \code{targetMuNormal}
and  \code{targetSigmaNormal}. The result of this calculation are two vectors of length \code{Ntests} containing the
Bhattacharyya for the test- and target-sample: \code{btestSample} and \code{btargetSample}. }
\item{Perform a statistical T-test for significant difference between the means of
\code{btestSample} and \code{btargetSample}.}
\item{Perform a Bonferoni p-value correction (multiplying the p-value by the number of single parameters or by the number
of parameter pairs.}
\item{If X is a single parameter with a significant Bonferoni-corrected p-value at the \code{tAlpha} critical level,
generate a combined histogram plot comparing the X-values in testSample against targetSample. Otherwise, if X is pair of
parameters with a significant Bonferoni-corrected p-value at the \code{tAlpha} critical level,
generate a combined 2d density plot, comparing the \code{X[1],X[2]}-values in testSample and targetSample.}
}
}
\examples{
\dontrun{
discr <- detectSampleDiscrepancy(
    system.file("extdata", "nlme_param_sampling", "PKparameters.xlsx",
                package = "IQRtools"),
    Npop = 500, Ntests = 10)
discr$histogramsDetectedSingleParams + ggplot2::theme(legend.position = "bottom")
discr$densplotsDetectedPairParams + ggplot2::theme(legend.position = "bottom")
}

}
\seealso{
\href{https://iqrtools.intiquan.com/doc/book/paramsampling.html}{Random sampling of NLME model parameters}

\href{https://iqrtools.intiquan.com/doc/book/GPF.html}{General Parameter Format}
}
\author{
Venelin Mitov
}
\keyword{internal}
