% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nlme_param_sampling.R
\name{sampleIndParamValues}
\alias{sampleIndParamValues}
\title{Sample individual parameter values, based on a NLME parameter
specification}
\usage{
sampleIndParamValues(
  spec,
  data = NULL,
  Nsamples = 1,
  Npop = if (FLAG_SAMPLE == 2) Nsamples else 1L,
  FLAG_SAMPLE = 0
)
}
\arguments{
\item{spec}{a filename (character string) denoting the path to a GPF
excel file, or a GPF object, or an IQRnlmeParamSpec object.}

\item{data}{a data.frame describing patient or other subject covariate and regressor data.
If nrow(data)==Nsamples, then all rows in data
are returned (in their original order) in the resulting data.frame; if
nrow(data)>Nsamples, sampling without replacement from the rows in data
is done; if nrow(data)<Nsamples then sampling with replacement is done.}

\item{Nsamples}{integer denoting the number of individuals in a
population. Note: in the case of \code{FLAG_SAMPLE=2}, this value is used to specify
the number of populations (see \code{Npop}). Default:
\code{if(is_IQRnlmeParamSpec(obj)) obj$Nsamples else 1L}.}

\item{Npop}{integer denoting the number of populations to sample. Default:
\code{if(is_IQRnlmeParamSpec(obj)) obj$Npop else if(FLAG_SAMPLE==2) Nsamples else 1L}.}

\item{FLAG_SAMPLE}{Flag indicating the type of sampling:
\tabular{ll}{
FLAG_SAMPLE       \tab Meaning \cr
===========       \tab =================================================== \cr
0                 \tab Use point estimates of population parameters (do not consider uncertainty) and sample Nsample
individual patients based on these. Covariates considered if defined by user and used in model.
Please note: population parameters do not take covariates into account!\cr
1                 \tab Sample single set of population parameters from uncertainty distribution and sample Nsample
individual patient parameters based on these. Covariates considered if defined by user and
used in model. Please note: population parameters do not take covariates into account!\cr
2                 \tab Sample Nsample sets of population parameters from uncertainty distribution.
Do not sample from variability distribution and do not take into account covariates (even
if user specified).\cr
3                 \tab Use point estimates of population parameters (do not consider uncertainty)
Return Nsamples sets of population parameters with covariates taken into account.\cr
4                 \tab Sample single set of population parameters from uncertainty distribution
Return Nsamples sets of population parameters with covariates taken into account.\cr
5                 \tab Use point estimates of population parameters (do not consider uncertainty) and sample Nsample
individual patients based on these. Covariates considered if defined by user and used in model.
Population parameters, typical individual parameters (no IIV, but considering covariates),
and individual parameters are returned.\cr
6                 \tab Sample single set of population parameters from uncertainty distribution and sample Nsample
individual patient parameters based on these. Covariates considered if defined by user and used in model.
Sampled population parameters, typical individual parameters (no IIV, but considering covariates),
and individual parameters are returned.\cr
7                 \tab !Supported in \code{\link{sample_IQRnlmeProject}} (Check note at the end)!
Same as FLAG_SAMPLE=0 but obtaining the empirical random effect covariance matrix from the post-hoc ETAs.
Only used for IQRsysProject. If IQRnlmeProject then fallback to flag 0. Note:
This option is not supported by sampleIndParamValues since post-hoc etas are not available in the GPF.
\cr
8                 \tab !Supported in \code{\link{sample_IQRnlmeProject}} (Check note at the end)!
Same as FLAG_SAMPLE=1 but obtaining the empirical random effect covariance matrix from the post-hoc ETAs.\cr
Only used for IQRsysProject. If IQRnlmeProject then fallback to flag 1. Note:
This option is not supported by sampleIndParamValues since post-hoc etas are not available in the GPF.
\cr
}}
}
\value{
a named list with the following elements:
\describe{
\item{\verb{$popParamValues}: }{ a data.frame with a column ID.POP and columns for all population parameter values.}
\item{\verb{$sampledData}: }{ a data.frame having \code{Nsample} rows representing sampled records from the patient data.}
\item{\verb{$typicalIndParamValues} : }{ a data.frame of \code{Nsample * nrow(popParamValues)} rows, columns \code{ID.POP} and \code{ID}, and columns
corresponding to the model parameters. The parameter values represent typical individual parameter values
taking into account the corresponding covariate values in \verb{$sampledData}.}
\item{\verb{$randomEffects} : }{ a data.frame of the same dimensionality as \verb{$typicalIndParamValues} containing the
random effects sampled from the IIV distribution (in normal units).}
\item{\verb{$indParamValues} : }{ a data.frame of the same dimensionality as \verb{$typicalIndParamValues} containing the (final)
individual parameter values in original units. }
\item{\verb{$FLAG_SAMPLE} : }{ same as the argument }
\item{\verb{$Nsamples} : }{ same as the argument }
}
}
\description{
Sample individual parameter values, based on a NLME parameter
specification
}
\details{
If \code{spec} is an IQRnlmeParamSpec object and \code{Npop} is specified,
the specified argument Npop overwrites the \code{spec$Npop}.
}
\examples{
\dontrun{
set.seed(1)
covrtData <- data.frame(
  ID = seq_len(4),
  WT0 = rlnorm(4, 3, sdlog = 0.2),
  SEX = sample(c(0, 1), 4, replace = TRUE))

fileGPF <- "material/01-07-PopulationSimulation/GPF/PKparametersNewFormat.xlsx"

# Read the xls file in GPF object: we can pass this object or fileGPF
# as a spec parameter to the sample* and calc* functions.
paramGPF <- read_GPFFromXLS(fileGPF)


set.seed(1)
listResultsSampling <- sampleIndParamValues(
  fileGPF,
  Nsamples = 10,
  FLAG_SAMPLE = 1,
  data = covrtData)

# create a IQRnlmeParamSpec object for future use (not suitable for reading by humans):
spec <- specifyParamSampling(fileGPF, Nsamples = 10, FLAG_SAMPLE = 1)
is_IQRnlmeParamSpec(spec)

set.seed(1)

popParamValues<- sampleParamFromUncertainty(spec)
# should be all 0s
listResultsSampling$popParamValues - popParamValues

sampledData <- sampleIDs(covrtData, spec$Nsamples)
# should be all 0s
listResultsSampling$sampledData - sampledData

typicalIndParamValues <- calcTypicalIndParamValues(spec, popParamValues, sampledData)
# should be all 0s
listResultsSampling$typicalIndParamValues - typicalIndParamValues

randomEffects <- sampleRandomEffects(spec, popParamValues)
# should be all 0s
listResultsSampling$randomEffects - randomEffects

indParamValues <- calcIndParamValues(spec, typicalIndParamValues, randomEffects)
# should be all 0s
listResultsSampling$indParamValues - indParamValues
}

}
\seealso{
\href{https://iqrtools.intiquan.com/doc/book/paramsampling.html}{Random sampling of NLME model parameters}

\href{https://iqrtools.intiquan.com/doc/book/GPF.html}{General Parameter Format}

Other Parameter sampling: 
\code{\link{calcIndParamValues}()},
\code{\link{calcTypicalIndParamValues}()},
\code{\link{combineGPF}()},
\code{\link{generate_GPFFromIQRnlmeEst}()},
\code{\link{generate_GPFFromIQRnlmeProject}()},
\code{\link{generate_GPFFromProjectInfoList}()},
\code{\link{generate_GPFFromProjectInfo}()},
\code{\link{getCorMatrixUncertainty}()},
\code{\link{getCovMatrixIIV}()},
\code{\link{getCovMatrixUncertainty}()},
\code{\link{getNamesAllParameters}()},
\code{\link{getNamesCovariateParameters}()},
\code{\link{getNamesCovariates}()},
\code{\link{getNamesErrorParameters}()},
\code{\link{getNamesIIVCorrParameters}()},
\code{\link{getNamesIIVParameters}()},
\code{\link{getNamesModelParameters}()},
\code{\link{getParamEstimates}()},
\code{\link{getParamTransformationTypes}()},
\code{\link{imputeTYPEcol_GPF}()},
\code{\link{is_GPF}()},
\code{\link{is_IQRnlmeParamSpec}()},
\code{\link{read_GPFFromCSV}()},
\code{\link{read_GPFFromXLS}()},
\code{\link{sampleIDs}()},
\code{\link{sampleParamFromUncertainty}()},
\code{\link{sampleRandomEffects}()},
\code{\link{sample_GPF}()},
\code{\link{select_FromGPF}()},
\code{\link{specifyParamSampling}()},
\code{\link{transformParamToNormal}()},
\code{\link{untransformParamFromNormal}()}
}
\author{
Venelin Mitov (IntiQuan)
}
\concept{Parameter sampling}
\keyword{internal}
