/*
 * cvodesIQRinterface.c: IQR Tools CVODES Interface for Sundials CVODES version 2.9.0
 *
 * Author: Henning Schmidt, IntiQuan GmbH
 * Email:  henning.schmidt@intiquan.com
 *
 * This code is published under the GNU GPL V3.
 *
 * (c) Copyright 2017, IntiQuan GmbH
 */

/* General header files */
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>

/* R Header Files */
#include <R.h>
#include <Rinternals.h>
#include <R_ext/Print.h>

/* Common header for C code model and CVODES interface function */
#include "cvodesIQRinterface.h"

/* CVODES Header Files */
#include "cvodes/cvodes/cvodes.h"
#include "cvodes/cvodes/cvodes_dense.h"
#include "cvodes/nvector/nvector_serial.h"
#include "cvodes/sundials/sundials_types.h"
#include "cvodes/sundials/sundials_math.h"

/* Define type of model function generated for each model */
typedef int model_func_type(double time_local, double *stateVector, double *DDTvector, ParamData *paramdataPtr, int DOflag, double *variableVector, double *reactionVector, double *gout, int *eventVector, DlsMat Jacs, N_Vector *yS);

/* Declare functions */
static int  f(double time, N_Vector u, N_Vector udot, void *modeldata);
static int  g(double time, N_Vector y, double *gout, void *modeldata);
static int  djac(long int N, double time, N_Vector y, N_Vector fy, DlsMat Jac, void *modeldata, N_Vector tmp1, N_Vector tmp2, N_Vector tmp3);
static int  fS(int Ns, double time, N_Vector y, N_Vector ydot, int iS, N_Vector yS, N_Vector ySdot, void *modeldata, N_Vector tmp1, N_Vector tmp2);
static int  simModel(int kstart, int kend);
static void addVec2Mat(double *matrix, double *rowvector, int row, int nrows, int ncols);
static void addVec2MatSelected(double *matrix, double *rowvector, int row, int nrows, int ncols, int *indices);
static void addSens2Mat(double *sensvalues, N_Vector *yS, int row);
static void calcAddOutputSens2Mat(double time, double *states, N_Vector *yS, double *outsensvalues, int row);
static void freeMem(int freeCVODE);
static void errorMsg(char *text);

/* Declare global variables related to IVP */
model_func_type *model_func;
double *timesimvector;
double *initialconditions;
double *parametervector;
double *resultXdot;
double *statevalues;
double *variablevalues;
double *variablevec;
double *reactionvalues;
double *reactionvec;
int    *eventvec;
N_Vector u;
void    *cvode_mem;
double *statevec;
int     NRSTATES;
int     NRPARAMETERS;
int     NRVARIABLES;
int     NRREACTIONS;
int     NREVENTS;
int     NRTIMESTEPS;
char    stringbuffer[256];
ParamData modeldata;
int     NREVENTTABLETIMESTEPS;
int    *ixEndTEventTablePieces;
double *dblMatrixEventTableParamValues;
double  treturn;

/* Declare additional global variables needed for sensitivity calculation */
int     FLAGsensitivity;
int     NRSENSPARAM;              /* NS in CVODES doc */
int    *indexParamSens;
N_Vector *yS;
int     FLAGuseSensEq;
int     opt_sens_simultaneous;
double *sensvalues;
double *sdata;
int     NRCOLOUTPUT;
double *initialconditionsSens;

/* Declare other control variables of interest */
int     FLAGoutputsOnly;
int    *outindices;
double *outvalues;
double *outvec;
int     NROUTPUTS;

/* Output sensitivities */
double *outsensvalues;
double *outsensvec;

/*
 *========================================================================
 * cvodesIQRinterface()
 * Main program - the IQR Tools CVODES interface
 *========================================================================
 */

SEXP cvodesIQRinterface(
  SEXP r_model_func_ptr,            /* Pointer to model function */
  SEXP r_timevector,                /* Double vector with time points for simulation */
  SEXP r_ICs,                       /* Double vector with initial conditions */

  /* Dosing related input arguments */
  SEXP r_ixEndTEventTablePieces,         /* Indices in time vector indicating events in the event table */
  SEXP r_dblMatrixEventTableParamValues, /* Parameter update value matrix - each row one event */

  /* Other control options for output */
  SEXP r_FLAGoutputsOnly,           /* Integer flag indicating to return only output information (1) or to return all (0) */
  SEXP r_outindices,                /* Integer vector with output indices in variables */

  /* Sensitivity analysis related input arguments */
  SEXP r_indexParamSens,            /* Integer vector with parameter indices used for sensitivity analysis */
  SEXP r_FLAGuseSensEq,             /* Integer flag to indicate to use explicit sensitivity equations (1) or not (0) */
  SEXP r_ICsensSim,                 /* Double vector with sensitivity initial conditions */
  SEXP r_opt_sens_simultaneous,     /* Integer flag to use CV_SIMULTANEOUS (1) or CV_STAGGERED (0) for sensitivity calculations */
  SEXP r_opt_sens_errcon,           /* Integer flag indicating if sensitivity variables included into error control strategy (1) or not (0) */
  SEXP r_opt_sens_maxnonlineariter, /* Integer value for maximum number of nonlinear solver iterations for sensitivity variables permitted per step */
  SEXP r_FLAGsensitivity,           /* Integer flag to indicate to determine sensitivities or not */

  /* Integrator options */
  SEXP r_opt_method_stiff,          /* Integer flag (0: non-stiff, 1:stiff) */
  SEXP r_opt_abstol,                /* Double value for absolute tolerance */
  SEXP r_opt_reltol,                /* Double value for relative tolerance */
  SEXP r_opt_minstep,               /* Double value for minimal integrator step-size */
  SEXP r_opt_maxstep,               /* Double value for maximal integrator step-size */
  SEXP r_opt_initstep,              /* Double value for initial step-size to be attempted */
  SEXP r_opt_maxnumsteps,           /* Integer value for maximum number of steps between two outputs */
  SEXP r_opt_maxerrtestfails,       /* Integer value for maximum number of error test failures in one step */
  SEXP r_opt_maxorder_stiff,        /* Integer value for maximum order of linear multistep method - STIFF */
  SEXP r_opt_maxorder_nonstiff,     /* Integer value for maximum order of linear multistep method - NONSTIFF */
  SEXP r_opt_maxconvfails,          /* Integer value for maximum number of nonlinear solver convergence failures in one step */
  SEXP r_opt_maxnonlineariter,      /* Integer value for maximum number of nonlinear solver iterations permitted per step */
  SEXP r_opt_usesymjac,             /* Integer flag to use symbolic jacobian */
  SEXP r_opt_xdotcalc,              /* Integer value defining what to do: 0=do integration, 1=return RHS of ODE for given time[0], states, parameters */
  SEXP r_verbose                    /* Integer flag for outputting additional diagnostic information */
) {


  /* ---------------------------------------------------------------------------
   Initialize memory pointers
   ----------------------------------------------------------------------------*/
  timesimvector                  = NULL;
  initialconditions              = NULL;
  parametervector                = NULL;
  resultXdot                     = NULL;
  statevalues                    = NULL;
  variablevalues                 = NULL;
  variablevec                    = NULL;
  reactionvalues                 = NULL;
  reactionvec                    = NULL;
  eventvec                       = NULL;
  u                              = NULL;
  cvode_mem                      = NULL;
  statevec                       = NULL;
  /* Dosing */
  ixEndTEventTablePieces         = NULL;
  dblMatrixEventTableParamValues = NULL;
  /* Sensitivity */
  indexParamSens                 = NULL;
  sensvalues                     = NULL;
  initialconditionsSens          = NULL;
  /* Output settings */
  outindices                     = NULL;
  outvalues                      = NULL;
  outvec                         = NULL;
  outsensvalues                  = NULL;
  outsensvec                     = NULL;

  /* ---------------------------------------------------------------------------
   Initialize and set scalar input argument variables
   ----------------------------------------------------------------------------*/
  /* r_model_func_ptr: User Model Function */
  model_func                    = (model_func_type *) R_ExternalPtrAddr(r_model_func_ptr);
  /* Define global parameters with info about the model */
  int     NRCOMPONENTS[5];
  model_func(0.0, NULL, NULL, NULL, DOFLAG_NRCOMPONENTS, NULL, NULL, NULL, &NRCOMPONENTS[0],NULL,NULL);
  NRSTATES                      = NRCOMPONENTS[0];
  NRPARAMETERS                  = NRCOMPONENTS[1];
  NRVARIABLES                   = NRCOMPONENTS[2];
  NRREACTIONS                   = NRCOMPONENTS[3];
  NREVENTS                      = NRCOMPONENTS[4];
  /* r_opt_*: CVODES integrator options */
  int    opt_method_stiff       = INTEGER(r_opt_method_stiff)[0];
  double opt_abstol             = REAL(r_opt_abstol)[0];
  double opt_reltol             = REAL(r_opt_reltol)[0];
  double opt_minstep            = REAL(r_opt_minstep)[0];
  double opt_maxstep            = REAL(r_opt_maxstep)[0];
  double opt_initstep           = REAL(r_opt_initstep)[0];
  int    opt_maxnumsteps        = INTEGER(r_opt_maxnumsteps)[0];
  int    opt_maxerrtestfails    = INTEGER(r_opt_maxerrtestfails)[0];
  int    opt_maxorder_stiff     = INTEGER(r_opt_maxorder_stiff)[0];
  int    opt_maxorder_nonstiff  = INTEGER(r_opt_maxorder_nonstiff)[0];
  int    opt_maxconvfails       = INTEGER(r_opt_maxconvfails)[0];
  int    opt_maxnonlineariter   = INTEGER(r_opt_maxnonlineariter)[0];
  /* r_opt_usesymjac: Flag if symbolic jacobian should be used */
  int    opt_usesymjac          = INTEGER(r_opt_usesymjac)[0];
  /* r_opt_xdotcalc: Integrator interface mode */
  int    opt_xdotcalc           = INTEGER(r_opt_xdotcalc)[0];
  /* r_verbose: verbose flag */
  int    verbose                = INTEGER(r_verbose)[0];
  /* Sensitivity options */
  opt_sens_simultaneous         = INTEGER(r_opt_sens_simultaneous)[0];
  int    opt_sens_errcon        = INTEGER(r_opt_sens_errcon)[0];
  int    opt_sens_maxnonlineariter = INTEGER(r_opt_sens_maxnonlineariter)[0];
  /* Other control options */
  FLAGoutputsOnly               = INTEGER(r_FLAGoutputsOnly)[0];

  /* --------------------------------------------------------------------------
   Dosing information variables
   ---------------------------------------------------------------------------*/
  NREVENTTABLETIMESTEPS             = LENGTH(r_ixEndTEventTablePieces);
  ixEndTEventTablePieces             = (int *) malloc(NREVENTTABLETIMESTEPS*sizeof(int));
  /* Get dimensions of r_dblMatrixEventTableParamValues and r_dblMatrixEventTableParamValues */
  SEXP dim                    = getAttrib( r_dblMatrixEventTableParamValues, R_DimSymbol ) ;
  /* Row dimension given by NREVENTTABLETIMESTEPS */
  dblMatrixEventTableParamValues        = (double *) malloc(NRPARAMETERS*NREVENTTABLETIMESTEPS*sizeof(double));
  for(int k=0; k<NREVENTTABLETIMESTEPS; k++) ixEndTEventTablePieces[k] = INTEGER(r_ixEndTEventTablePieces)[k]-1; /* Substract one for C */
  for(int k=0; k<NRPARAMETERS*NREVENTTABLETIMESTEPS; k++) dblMatrixEventTableParamValues[k] = REAL(r_dblMatrixEventTableParamValues)[k];

  /* --------------------------------------------------------------------------
   Sensitivity analysis related variables
   ---------------------------------------------------------------------------*/
  FLAGsensitivity               = INTEGER(r_FLAGsensitivity)[0];
  if (FLAGsensitivity) {
    FLAGuseSensEq               = INTEGER(r_FLAGuseSensEq)[0];
    NRSENSPARAM                 = LENGTH(r_indexParamSens);
    indexParamSens              = (int *) malloc(NRSENSPARAM*sizeof(int));
    for(int k=0; k<NRSENSPARAM; k++) indexParamSens[k] = INTEGER(r_indexParamSens)[k]-1; /* Substract one for C */
  initialconditionsSens       = (double *) malloc(NRSTATES*NRSENSPARAM*sizeof(double));
  for(int k=0; k<NRSTATES*NRSENSPARAM; k++) initialconditionsSens[k] = REAL(r_ICsensSim)[k];
  }

  /* --------------------------------------------------------------------------
   Output related variables
   ---------------------------------------------------------------------------*/
  if (FLAGoutputsOnly) {
    NROUTPUTS                   = LENGTH(r_outindices);
    outindices                  = (int *) malloc(NROUTPUTS*sizeof(int));
    for(int k=0; k<NROUTPUTS; k++) outindices[k] = INTEGER(r_outindices)[k]-1; /* Substract one for C */
  }

  /* ---------------------------------------------------------------------------
   Initialize and set vector input argument variables (requires memory allocation)
   ----------------------------------------------------------------------------*/
  NRTIMESTEPS                   = LENGTH(r_timevector);
  timesimvector                 = (double *) malloc(NRTIMESTEPS*sizeof(double));
  initialconditions             = (double *) malloc(NRSTATES*sizeof(double));
  parametervector               = (double *) malloc(NRPARAMETERS*sizeof(double));
  for(int k=0; k<LENGTH(r_timevector); k++) timesimvector[k] = REAL(r_timevector)[k];
  for(int k=0; k<NRSTATES; k++) initialconditions[k] = REAL(r_ICs)[k];
  /* Initialize parametervalues variable */
  modeldata.parametervector = parametervector; /* Pointer to allocated space */
  /* Fill parametervalues with initial values from the event table (first row) */
  for (int k2=0; k2<NRPARAMETERS; k2++) {
    modeldata.parametervector[k2] = dblMatrixEventTableParamValues[NREVENTTABLETIMESTEPS*k2];
  }

  /* ---------------------------------------------------------------------------
   Diagnostic output in "verbose" mode
   ----------------------------------------------------------------------------*/
  if (verbose) {
    Rprintf("--------------------------------------\n");
    Rprintf("IQR Tools CVODES Integrator Interface!\n");
    Rprintf("--------------------------------------\n");
    Rprintf("\n");
    Rprintf("Model Information:\n");
    Rprintf("------------------\n");
    Rprintf("NRSTATES:                %d\n",NRSTATES);
    Rprintf("NRPARAMETERS:            %d\n",NRPARAMETERS);
    Rprintf("NRVARIABLES:             %d\n",NRVARIABLES);
    Rprintf("NRREACTIONS:             %d\n",NRREACTIONS);
    Rprintf("NREVENTS:                %d\n",NREVENTS);
    Rprintf("\n");
    Rprintf("General settings:\n");
    Rprintf("-----------------------\n");
    Rprintf("opt_xdotcalc:            %d\n",opt_xdotcalc);
    Rprintf("verbose:                 %d\n",opt_xdotcalc);
    Rprintf("\n");
    Rprintf("CVODES options:\n");
    Rprintf("---------------\n");
    Rprintf("opt_method_stiff:        %d\n",opt_method_stiff);
    Rprintf("opt_abstol:              %f\n",opt_abstol);
    Rprintf("opt_reltol:              %f\n",opt_reltol);
    Rprintf("opt_minstep:             %f\n",opt_minstep);
    Rprintf("opt_maxstep:             %f\n",opt_maxstep);
    Rprintf("opt_initstep:            %f\n",opt_initstep);
    Rprintf("opt_maxnumsteps:         %d\n",opt_maxnumsteps);
    Rprintf("opt_maxerrtestfails:     %d\n",opt_maxerrtestfails);
    Rprintf("opt_maxorder_stiff:      %d\n",opt_maxorder_stiff);
    Rprintf("opt_maxorder_nonstiff:   %d\n",opt_maxorder_nonstiff);
    Rprintf("opt_maxconvfails:        %d\n",opt_maxconvfails);
    Rprintf("opt_maxnonlineariter:    %d\n",opt_maxnonlineariter);
    Rprintf("\n");
  }

  /* ---------------------------------------------------------------------------
   Initialize additional variables for output and model information
   ----------------------------------------------------------------------------*/
  SEXP output;                                  /* Output variable */
  int rootdir[1];                               /* Root finding direction */

  /* ---------------------------------------------------------------------------
   Handle calculation of ODE RHS if desired
   ----------------------------------------------------------------------------*/
  if (opt_xdotcalc) {
    resultXdot = (double *) malloc(NRSTATES*sizeof(double));
    /* Call the model for evaluation of Xdot */
    model_func(timesimvector[0], initialconditions, resultXdot, &modeldata, DOFLAG_DDT, NULL, NULL, NULL, NULL, NULL, NULL);
    /* Define R output variable */
    PROTECT (output = allocVector(REALSXP, NRSTATES));
    for (int k=0; k<NRSTATES; k++) REAL(output)[k] = resultXdot[k];
    UNPROTECT(1);
    /* Free memory */
    freeMem(0);
    /* Return results to R */
    return(output);
  }

  /* ---------------------------------------------------------------------------
   Allocate simulation memory
   ----------------------------------------------------------------------------*/
  /* Always allocate variablevec and reactionvec memory */
  if (NRVARIABLES > 0) variablevec = (double *) malloc(NRVARIABLES*sizeof(double));
  if (NRREACTIONS > 0) reactionvec = (double *) malloc(NRREACTIONS*sizeof(double));

  if (!FLAGoutputsOnly) {
    /* Skipping returning of states, variables, reactions and state sensitivities saves memory */
    statevalues = (double *) malloc(NRTIMESTEPS*NRSTATES*sizeof(double));
    if (NRVARIABLES > 0) variablevalues  = (double *) malloc(NRTIMESTEPS*NRVARIABLES*sizeof(double));
    if (NRREACTIONS > 0) reactionvalues  = (double *) malloc(NRTIMESTEPS*NRREACTIONS*sizeof(double));
  } else {
    /* Only get memory for output information */
    outvalues         = (double *) malloc(NRTIMESTEPS*NROUTPUTS*sizeof(double));
    outvec            = (double *) malloc(NROUTPUTS*sizeof(double));
  }
  /* Get memory for events */
  if (NREVENTS > 0) {
    eventvec          = (int *)    malloc(NREVENTS*sizeof(int));
  }

  /* ---------------------------------------------------------------------------
   Initialize the integrator for the IVP
   ----------------------------------------------------------------------------*/
  u = N_VMake_Serial(NRSTATES,initialconditions);
  /* Set integration method and maxorder (stiff or non-stiff) */
  if (opt_method_stiff == 1) {
    /* stiff */
    cvode_mem = CVodeCreate(CV_BDF, CV_NEWTON);
    CVodeSetMaxOrd(cvode_mem, opt_maxorder_stiff);
  }
  else {
    /* nonstiff */
    cvode_mem = CVodeCreate(CV_ADAMS, CV_FUNCTIONAL);
    CVodeSetMaxOrd(cvode_mem, opt_maxorder_nonstiff);
  }
  /* Initialize CVODE */
  CVodeInit(cvode_mem, f, timesimvector[0], u);
  treturn = timesimvector[0];
  /* Set tolerances */
  CVodeSStolerances(cvode_mem, opt_reltol, opt_abstol);
  /* Set Jacobian information */
  CVDense(cvode_mem,NRSTATES);
  if (opt_usesymjac) {
    /* Provide user defined jacobian function - if desired */
    CVDlsSetDenseJacFn(cvode_mem,djac);
  }
  /* Initialize root finding */
  if (NREVENTS > 0) {
    CVodeRootInit(cvode_mem, NREVENTS, g);
    /* Set direction of root finding to increasing through 0 */
    rootdir[0] = 1;
    CVodeSetRootDirection(cvode_mem,&rootdir[0]);
  }
  /* Set model data information */
  CVodeSetUserData(cvode_mem, &modeldata);
  /* Set additional integrator options */
  if (opt_minstep > 0) CVodeSetMinStep(cvode_mem, opt_minstep);
  if (opt_maxstep > 0) CVodeSetMaxStep(cvode_mem, opt_maxstep);
  if (opt_maxnumsteps > 0) CVodeSetMaxNumSteps(cvode_mem, opt_maxnumsteps);
  CVodeSetMaxErrTestFails(cvode_mem, opt_maxerrtestfails);
  CVodeSetMaxConvFails(cvode_mem, opt_maxconvfails);
  CVodeSetInitStep(cvode_mem, opt_initstep);
  CVodeSetMaxNonlinIters(cvode_mem, opt_maxnonlineariter);

  /* ---------------------------------------------------------------------------
   Initialize the sensitivity analysis
   ----------------------------------------------------------------------------*/
  if (FLAGsensitivity) {
    /* pbar: Define "NS" positive scaling factors */
    double pbar[NRSENSPARAM];
    for (int k=0; k<NRSENSPARAM; k++) {
      /* Index into parameters by indexParamSens */
      double value = modeldata.parametervector[indexParamSens[k]];
      /* Handle 0 parameters and ensure positiveness */
      if (value == 0.0)
        pbar[k] = 1.0;
      else
        pbar[k] = fabs(value);
    }
    /* plist: Already defined by "indexParamSens" */
    /* p: probably already defined as "modeldata.parametervector" ... lets try */

    /* Create sensitivity array */
    yS = N_VCloneVectorArray_Serial(NRSENSPARAM,u);

    /* Set sensitivity ICs */
    for (int kpar=0; kpar<NRSENSPARAM; kpar++) {
      sdata = N_VGetArrayPointer_Serial(yS[kpar]);
      for (int kstate=0; kstate<NRSTATES; kstate++) {
        sdata[kstate] = initialconditionsSens[kstate*NRSENSPARAM + kpar];
      }
    }

    /* Activate sensitivity calculations */
    if (FLAGuseSensEq) {
      if (opt_sens_simultaneous) CVodeSensInit1(cvode_mem, NRSENSPARAM, CV_SIMULTANEOUS, fS, yS);
      else CVodeSensInit1(cvode_mem, NRSENSPARAM, CV_STAGGERED, fS, yS);
    } else {
      if (opt_sens_simultaneous) CVodeSensInit(cvode_mem, NRSENSPARAM, CV_SIMULTANEOUS, NULL, yS);
      else CVodeSensInit(cvode_mem, NRSENSPARAM, CV_STAGGERED, NULL, yS);
    }

    /* Set sensitivity tolerances - for now lets CVODES estimate reasonable ones */
    CVodeSensEEtolerances(cvode_mem);

    /* Set sensitivity parameters */
    if (FLAGuseSensEq) CVodeSetSensParams(cvode_mem, NULL, pbar, NULL);
    else CVodeSetSensParams(cvode_mem, modeldata.parametervector, pbar, indexParamSens);

    /* Error control settings for sensitivity variables */
    if (opt_sens_errcon) CVodeSetSensErrCon(cvode_mem,TRUE);
    else CVodeSetSensErrCon(cvode_mem,FALSE);

    /* Set max nonlinear iterations for sensitivity calculations */
    CVodeSetSensMaxNonlinIters(cvode_mem,opt_sens_maxnonlineariter);

    if (!FLAGoutputsOnly) {
      /* Allocate sensitivity result memory */
      sensvalues        = (double *) malloc(NRTIMESTEPS*NRSTATES*NRSENSPARAM*sizeof(double));
      /* Store sensitivity initial conditions */
      addSens2Mat(sensvalues, yS, 0);
    } else {
      /* Allocate memory for output sensitivities */
      outsensvalues     = (double *) malloc(NRTIMESTEPS*NROUTPUTS*NRSENSPARAM*sizeof(double));
      outsensvec        = (double *) malloc(NROUTPUTS*NRSENSPARAM*sizeof(double));

      /* Calc and evaluate Output Sensitivities ICs */
      calcAddOutputSens2Mat(timesimvector[0],initialconditions,yS,outsensvalues,0);
    }
  }

  /* ---------------------------------------------------------------------------
   Handle intial conditions
   ----------------------------------------------------------------------------*/

  /* Evaluate variables and reactions for initial conditions */
  model_func(timesimvector[0], initialconditions, NULL, &modeldata, DOFLAG_VARREAC, variablevec, reactionvec,NULL,NULL,NULL, NULL);

  if (!FLAGoutputsOnly) {
    /* Add initial conditions to statevalues matrix */
    addVec2Mat(statevalues,initialconditions,0,NRTIMESTEPS,NRSTATES);
    /* Add variables and reactions to corresponding matrices */
    if (NRVARIABLES > 0) addVec2Mat(variablevalues,variablevec,0,NRTIMESTEPS,NRVARIABLES);
    if (NRREACTIONS > 0) addVec2Mat(reactionvalues,reactionvec,0,NRTIMESTEPS,NRREACTIONS);
  } else {
    /* Handle storing output information */
    if (NROUTPUTS > 0)   addVec2MatSelected(outvalues,variablevec,0,NRTIMESTEPS,NROUTPUTS,outindices);
  }

  /* ---------------------------------------------------------------------------
   Do the integration
   ----------------------------------------------------------------------------*/

  int     kstart; /* Index of starting time step */
  int     kend;   /* Index of end time step for single simulation */

  /* NREVENTTABLETIMESTEPS is one larger than the number of actual events ... the last time is the final
   * simulation time ... */
  for (int k=0; k<NREVENTTABLETIMESTEPS-1; k++) {
    /* Define start and end time points for next step */
    kstart = ixEndTEventTablePieces[k];
    kend   = ixEndTEventTablePieces[k+1];

    /* Update model parameters from the event Table */
    for (int k2=0; k2<NRPARAMETERS; k2++) {
      if (!ISNA(dblMatrixEventTableParamValues[k+NREVENTTABLETIMESTEPS*k2])) {
        modeldata.parametervector[k2] = dblMatrixEventTableParamValues[k+NREVENTTABLETIMESTEPS*k2];
      }
    }

    /* Need to reinit the integrator */
    CVodeReInit(cvode_mem, treturn, u);

    /* Need to reinit the sensitivity computation */
    if (FLAGsensitivity) {
      if (opt_sens_simultaneous) CVodeSensReInit(cvode_mem, CV_SIMULTANEOUS, yS);
      else CVodeSensReInit(cvode_mem, CV_STAGGERED, yS);
    }

    /* Do the simulation */
    simModel(kstart,kend);
  }


  /* ---------------------------------------------------------------------------
   Create R output variable
   ----------------------------------------------------------------------------*/
  int NRSENSPARAMSTATES;
  int NRSENSPARAMOUT;

  if (FLAGsensitivity) {
    NRSENSPARAMSTATES = NRSTATES*NRSENSPARAM;
    NRSENSPARAMOUT    = NROUTPUTS*NRSENSPARAM;
  } else {
    NRSENSPARAMSTATES = 0;
    NRSENSPARAMOUT    = 0;
  }

  if (FLAGoutputsOnly) {
    NRSTATES          = 0;
    NRVARIABLES       = 0;
    NRREACTIONS       = 0;
    NRSENSPARAMSTATES = 0;
  } else {
    NROUTPUTS         = 0;
    NRSENSPARAMOUT    = 0;
  }

  if (FLAGsensitivity) NRCOLOUTPUT = 1+NROUTPUTS+NRSENSPARAMOUT+NRSTATES+NRVARIABLES+NRREACTIONS+NRSENSPARAMSTATES;
  else NRCOLOUTPUT = 1+NROUTPUTS+NRSTATES+NRVARIABLES+NRREACTIONS;

  PROTECT (output = allocMatrix(REALSXP, NRTIMESTEPS, NRCOLOUTPUT));
  for (int krow=0; krow<NRTIMESTEPS; krow++) {
    /* Copy time */
    REAL(output)[krow] = timesimvector[krow];

    /* Copy output information */
    for (int kcol=0; kcol<NROUTPUTS; kcol++) {
      REAL(output)[krow + NRTIMESTEPS*(kcol+1)] = outvalues[krow + NRTIMESTEPS*kcol];
    }

    /* Copy output sensitivity information */
    for (int kcol=0; kcol<NRSENSPARAMOUT; kcol++) {
      REAL(output)[krow + NRTIMESTEPS*(kcol+NROUTPUTS+1)] = outsensvalues[krow + NRTIMESTEPS*kcol];
    }

    /* Copy state information */
    for (int kcol=0; kcol<NRSTATES; kcol++) {
      REAL(output)[krow + NRTIMESTEPS*(kcol+NROUTPUTS+NRSENSPARAMOUT+1)] = statevalues[krow + NRTIMESTEPS*kcol];
    }

    /* Copy variable information */
    for (int kcol=0; kcol<NRVARIABLES; kcol++) {
      REAL(output)[krow + NRTIMESTEPS*(kcol+NROUTPUTS+NRSENSPARAMOUT+NRSTATES+1)] = variablevalues[krow + NRTIMESTEPS*kcol];
    }

    /* Copy reaction information */
    for (int kcol=0; kcol<NRREACTIONS; kcol++) {
      REAL(output)[krow + NRTIMESTEPS*(kcol+NROUTPUTS+NRSENSPARAMOUT+NRSTATES+NRVARIABLES+1)] = reactionvalues[krow + NRTIMESTEPS*kcol];
    }

    if (FLAGsensitivity) {
      /* Copy sensitivity information */
      for (int kcol=0; kcol<NRSENSPARAMSTATES; kcol++) {
        REAL(output)[krow + NRTIMESTEPS*(kcol+NROUTPUTS+NRSENSPARAMOUT+NRSTATES+NRVARIABLES+NRREACTIONS+1)] = sensvalues[krow + NRTIMESTEPS*kcol];
      }
    }

  }
  UNPROTECT(1);

  /* ---------------------------------------------------------------------------
   Free allocated memory
   ----------------------------------------------------------------------------*/
  freeMem(1);

  /* ---------------------------------------------------------------------------
   Free memory
   ----------------------------------------------------------------------------*/
  return(output);
}


/*
 *========================================================================
 * Main CVODE simulation function
 *========================================================================
 */
static int simModel(int kstart, int kend)
{
  int     ktime           = kstart+1;             /* First time step for next value */
double  tendstep        = timesimvector[ktime]; /* Next stop time of integrator */
int     flag_CVODE      = 0;                    /* CVODE return flag */

/* Loop integration logistics until ready */
while(1) {

  /* Run integration */
  flag_CVODE = CVode(cvode_mem, tendstep, u, &treturn, CV_NORMAL);

  /* Check CVODE return flag for potential issues */
  if (flag_CVODE < 0) {
    if (flag_CVODE == CV_TOO_MUCH_WORK) errorMsg("CVODE Error: CV_TOO_MUCH_WORK");
    else if (flag_CVODE == CV_TOO_MUCH_ACC) errorMsg("CVODE Error: CV_TOO_MUCH_ACC");
    else if (flag_CVODE == CV_ERR_FAILURE || flag_CVODE == CV_CONV_FAILURE) errorMsg("CVODE Error: CV_ERR_FAILURE");
    else {
      sprintf(stringbuffer, "CVODE Error Flag: %d",flag_CVODE);
      errorMsg(stringbuffer);
    }
  }

  /* Get pointer to updated state vector */
  statevec = NV_DATA_S(u);

  /* Check if event(s) happened */
  if (flag_CVODE == CV_ROOT_RETURN) {

    /*
     * Test code for assessment of sensitivities when an event has fired ...
     * it shows that if the event time is not in the simtime vector then
     * CVODES f.cks up and the sensitivities at that point are not correct.
     * If the time is in the simtime vector then the sensitivities are correct.
     * Tested against same model in dMod ... no solution at the moment. Except that
     * if you want to be safe with sensitivities: do not use events!
    for (int kkk=0; kkk<NRSTATES; kkk++) {
      for (int kkk2=0; kkk2<NRSENSPARAM; kkk2++) {
        Rprintf("state %d, param %d: %f\n",kkk,kkk2,N_VGetArrayPointer_Serial(yS[kkk2])[kkk]);
      }
    }
     */

    /* Event happened - get event information from CVODE */
    CVodeGetRootInfo(cvode_mem, eventvec);

    /* Do event assignment */
    model_func(treturn, statevec, NULL, &modeldata, DOFLAG_EVENTASSIGN, NULL, NULL, NULL, eventvec,NULL, NULL);

    /* Re-initialize CVODE integrator */
    flag_CVODE = CVodeReInit(cvode_mem, treturn, u);

    /* Need to reinit the sensitivity computation */
    if (FLAGsensitivity) {
      /*
       Do sensitivity states event assignment - we only handle setting to 0 at the moment
       statevec passed but not needed ... it is read out in the model ... therefor.
       */
      model_func(treturn, statevec, NULL, &modeldata, DOFLAG_EVENTASSIGN_SENS, NULL, NULL, NULL, eventvec,NULL, yS);

      if (opt_sens_simultaneous) CVodeSensReInit(cvode_mem, CV_SIMULTANEOUS, yS);
      else CVodeSensReInit(cvode_mem, CV_STAGGERED, yS);
    }

  }

  /* Handle case where simulator reached the end time for a particular step */
  if (tendstep == treturn) {

    /* Evaluate variables and reactions for state */
    model_func(tendstep, statevec, NULL, &modeldata, DOFLAG_VARREAC, variablevec, reactionvec, NULL, NULL,NULL, NULL);

    if (!FLAGoutputsOnly) {
      /* Add state results to statevalues matrix */
      addVec2Mat(statevalues,statevec,ktime,NRTIMESTEPS,NRSTATES);
      /* Add variables and reactions to corresponding matrices */
      if (NRVARIABLES > 0) addVec2Mat(variablevalues,variablevec,ktime,NRTIMESTEPS,NRVARIABLES);
      if (NRREACTIONS > 0) addVec2Mat(reactionvalues,reactionvec,ktime,NRTIMESTEPS,NRREACTIONS);
    } else {
      /* Handle storing output information */
      if (NROUTPUTS > 0)
        addVec2MatSelected(outvalues,variablevec,ktime,NRTIMESTEPS,NROUTPUTS,outindices);
    }

    /* Get sensitivities, if calculated */
    if (FLAGsensitivity) {
      CVodeGetSens(cvode_mem,&treturn,yS);

      if (!FLAGoutputsOnly) {
        /* Store sensitivity results */
        addSens2Mat(sensvalues, yS, ktime);
      } else {
        /* Output sensitivities */
        calcAddOutputSens2Mat(tendstep,statevec,yS,outsensvalues,ktime);
      }
    }

    /* Go to next time step and stop integration if last time reached */
    ktime = ktime+1;
    if (ktime <= kend)
      tendstep = timesimvector[ktime];
    else {
      break;
    }
  }
}
return(0);
}


/*
 *========================================================================
 * RHS function f(t,u)
 *========================================================================
 */
static int f(double time, N_Vector u, N_Vector udot, void *modeldata)
{
  double *statevec, *DDTvector;
  ParamData *paramdataPtr;
  /* get pointer to modeldata */
  paramdataPtr = (ParamData*) modeldata;
  /* connect input and result data */
  statevec = NV_DATA_S(u);
  DDTvector = NV_DATA_S(udot);
  /* run the model */
  model_func(time, statevec, DDTvector, paramdataPtr, DOFLAG_DDT, NULL, NULL, NULL, NULL,NULL, NULL);
  return(0);
}


/*
 *========================================================================
 * Event function
 *========================================================================
 */
static int g(double time, N_Vector y, double *gout, void *modeldata)
{
  double *statevec;
  ParamData *paramdataPtr;
  /* get pointer to model data */
  paramdataPtr = (ParamData*) modeldata;
  /* connect input data */
  statevec = NV_DATA_S(y);
  /* run the event function */
  model_func(time, statevec, NULL, paramdataPtr, DOFLAG_EVENTS, NULL, NULL, gout, NULL,NULL, NULL);
  return(0);
}

/*
 *========================================================================
 * Dense Jacobian function
 *========================================================================
 */
static int  djac(long int N, double time, N_Vector y, N_Vector fy, DlsMat Jac, void *modeldata, N_Vector tmp1, N_Vector tmp2, N_Vector tmp3)
{
  double *statevec = NV_DATA_S(y);
  double *ddtvec   = NV_DATA_S(fy);
  ParamData *paramdataPtr;
  paramdataPtr = (ParamData*) modeldata;
  model_func(time, statevec, ddtvec, paramdataPtr, DOFLAG_JACOBIAN, NULL, NULL, NULL, NULL,Jac, NULL);
  return(0);
}

/*
 *========================================================================
 * Sensitivitiy RHS
 * It must compute the vector (df/dy)si(t) + (df/dp) for i = iS and store it in ySdot.
 *========================================================================
 */
static int  fS(int Ns, double time, N_Vector y, N_Vector ydot, int iS, N_Vector yS, N_Vector ySdot, void *modeldata, N_Vector tmp1, N_Vector tmp2) {
  double *states          = NV_DATA_S(y);     /* input */
double *sensstates      = NV_DATA_S(yS);    /* input */
double *ddtsensstates   = NV_DATA_S(ySdot); /* output */
ParamData *paramdataPtr;
paramdataPtr = (ParamData*) modeldata;
model_func(time, states, sensstates, paramdataPtr, DOFLAG_SENSITIVITY, ddtsensstates, NULL, NULL, &iS, NULL, NULL);
return(0);
}

/*
 *========================================================================
 * Help function to add a row vector to a matrix
 *========================================================================
 */
static void addVec2Mat(double *matrix, double *rowvector, int row, int nrows, int ncols)
{
  int k;
  for (k=0;k<ncols;k++) {
    matrix[row+k*nrows] = rowvector[k];
  }
}

/*
 *========================================================================
 * Help function to add indexed elements of a vector into row of a matrix
 *========================================================================
 */
static void addVec2MatSelected(double *matrix, double *rowvector, int row, int nrows, int ncols, int *indices)
{
  int k;
  for (k=0;k<ncols;k++) {
    matrix[row+k*nrows] = rowvector[indices[k]];
  }
}

/*
 *========================================================================
 * Help function to store sensitivity results to a matrix
 * df1/dp1---df1/dpn, ..., ... dfm/dp1---dfm/dpn
 *========================================================================
 */
static void addSens2Mat(double *sensvalues, N_Vector *yS, int row)
{
  int col, kpar, kstate;
  for (kpar=0; kpar<NRSENSPARAM; kpar++) {
    sdata = N_VGetArrayPointer_Serial(yS[kpar]);
    for (kstate=0; kstate<NRSTATES; kstate++) {
      col = kstate*NRSENSPARAM + kpar;
      sensvalues[row+col*NRTIMESTEPS] = sdata[kstate];
    }
  }
}

/*
 *========================================================================
 * Help function to calculate output sensitivities and store them in a matrix
 * dg1/dp1---dg1/dpn, ..., ... dgm/dp1---dgm/dpn
 *========================================================================
 */
static void calcAddOutputSens2Mat(double time, double *states, N_Vector *yS, double *outsensvalues, int row)
{
  double   *sensstates;
  int       kpar, kout, col;

  for (kpar=0; kpar<NRSENSPARAM; kpar++) {
    /* Calculate output sensitivities */
    model_func(time, states, NV_DATA_S(yS[kpar]), &modeldata, DOFLAG_OUTSENS, outsensvec, NULL, NULL, &kpar, NULL, NULL);
    /* Store output sensitivities */
    for (kout=0; kout<NROUTPUTS; kout++) {
      col = kout*NRSENSPARAM + kpar;
      outsensvalues[row+col*NRTIMESTEPS] = outsensvec[kout];
    }
    //    Rprintf("%f\n",outsensvec[kpar]);

  }
  //  Rprintf("\n");
}

/*
 *========================================================================
 * Free the memory
 *========================================================================
 */
static void freeMem(int freeCVODE)
{
  /* Free all the malloc allocated memory */
  if(timesimvector != NULL)     free(timesimvector);
  if(initialconditions != NULL) free(initialconditions);
  if(parametervector != NULL)   free(parametervector);
  if(resultXdot != NULL)        free(resultXdot);

  /* Free all dosing related allocations pre Xdot calc */
  if(ixEndTEventTablePieces!=NULL)       free(ixEndTEventTablePieces);
  if(dblMatrixEventTableParamValues!=NULL)  free(dblMatrixEventTableParamValues);

  /* Free all sensitivity related allocations pre Xdot calc */
  if (FLAGsensitivity) {
    if(indexParamSens!=NULL)        free(indexParamSens);
  }

  /* Free all CVODE related memory post Xdot calculation */
  if (freeCVODE) {
    if(variablevec != NULL)         free(variablevec);
    if(reactionvec != NULL)         free(reactionvec);
    if (!FLAGoutputsOnly) {
      if(statevalues != NULL)       free(statevalues);
      if(variablevalues != NULL)    free(variablevalues);
      if(reactionvalues != NULL)    free(reactionvalues);
    } else {
      if(outvalues != NULL)         free(outvalues);
      if(outvec != NULL)            free(outvec);
    }
    if(eventvec != NULL)            free(eventvec);

    N_VDestroy_Serial(u);   /* Free the u vector */
  CVodeFree(&cvode_mem);  /* Free the integrator memory */
  if (FLAGsensitivity) {
    /* Free additionally sensitivity related memory */
    if (!FLAGoutputsOnly) {
      if (sensvalues != NULL)     free(sensvalues);
    }
    N_VDestroyVectorArray_Serial(yS, NRSENSPARAM);
  }
  }
}

/*
 *========================================================================
 * Error function - also freeing memory - and exiting to R
 *========================================================================
 */
static void errorMsg(char *text)
{
  /* Free allocated memory */
  freeMem(1);

  /* Then print error message and exit */
  error(text);
}


